
# INFO  ############################################################
# 23-05-2023 PM2,5 de 2022
# Substitue all NA's in df_obs_sub by gbm predictions based on ok_total.


# if we imported observations from ghost, the stations names have suffixes and prefixes
# we want to remove them
names(df_obs_sub) <- names(df_obs_sub) %>%
  gsub("STA_","",.) %>% gsub("\\_.*","",.) %>% gsub("STA-","",.)


# Load All observations #######################################################
df_obs_years        <- NULL
for(read_year in years_gbm){
  
  if(read_year==2018){
    df_obs_gbm        <- readRDS(paste0(read_data_gbm,read_year,"/results_gene_",pollutant, "/output/","df_obs_ghost_",pollutant,".rds"))
  }else{
    df_obs_gbm        <- readRDS(paste0(read_data_gbm,read_year,"/results_gene_",pollutant, "/output/","df_obs_gene_",pollutant,".rds"))
  }
  
  # df_obs_gbm        <- readRDS(file = paste0(read_data_gbm,read_year,"/results_gene_",pollutant,"/df_obs_gene_",pollutant,".rds"))
  
  start_date_gbm  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date_gbm    <- as.Date(paste(read_year, "-12-31", sep = ""))
  
  dates_gbm      <- seq(start_date_gbm, end_date_gbm, by = "day")
  df_obs_sub_gbm <- subset(df_obs_gbm, date>=start_date_gbm & date <= end_date_gbm)
  
  
  # Check we don't have duplicated obs
  if(any(duplicated(df_obs_sub_gbm$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub_gbm$date[duplicated(df_obs_sub_gbm$date)]))
    df_obs_sub_gbm <- distinct(df_obs_sub_gbm, date, .keep_all = TRUE)
    print("The df_obs_sub (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub_gbm$date) != length(dates_gbm)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  # if we imported some data from ghost, the stations names have suffixes and prefixes
  # we want to remove them
  names(df_obs_sub_gbm) <- names(df_obs_sub_gbm) %>%
    gsub("STA_","",.) %>% gsub("\\_.*","",.) %>% gsub("STA-","",.)
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub_gbm)
  
  # subset only stations active during the selected year
  df_obs_years <- df_obs_years %>%
    dplyr::select(intersect(colnames(df_obs_sub) , 
                            colnames(df_obs_years)))
}


# remove observations from portugal and france
df_obs_years <-  df_obs_years %>% 
  dplyr::select(date,starts_with("ES"))
  
# convert NANs to NAs
df_obs_years <- df_obs_years %>% 
  mutate(across(where(is.numeric), ~ ifelse(is.nan(.), NA, .)))


# remove prefixes and sufixes of ghost data station names
if(file_format=="ghost"){
  local_stations@data$ref_obs <- local_stations@data$ref_obs %>%
    gsub("STA_","",.) %>% gsub("\\_.*","",.) %>% gsub("STA-","",.)
}


names_local_stations <- as.list(intersect(local_stations@data$ref_obs,
                                          colnames(df_obs_years)))

# Load raw model interpolated to AQMS #########################################


# df_all_model_raw_at_AQMS_gbm <- NULL
for(read_year in years_gbm){

  if(read_year==2018){
    read_model_raw_at_AQMS_df <- readRDS(paste0(read_data_gbm,read_year,"/results_gene_",pollutant, "/output/model_raw_at_AQMS_ghost_",pollutant,"_df.rds"))
  }else{
    read_model_raw_at_AQMS_df <- readRDS(paste0(read_data_gbm,read_year,"/results_gene_",pollutant, "/output/model_raw_at_AQMS_gene_",pollutant,"_df.rds"))
  }
  
  if(time_resolution=="daily"){
    read_model_raw_at_AQMS_df$date <- as.Date(read_model_raw_at_AQMS_df$date, "%Y-%m-%d", tz="UTC")
  }
  
  # if we imported some data from ghost, the stations names have suffixes and prefixes
  # we want to remove these
  names(read_model_raw_at_AQMS_df) <- names(read_model_raw_at_AQMS_df) %>%
    gsub("STA_","",.) %>% gsub("\\_.*","",.) %>% gsub("STA-","",.)
  
  if (exists("df_all_model_raw_at_AQMS_gbm")) {
    df_all_model_raw_at_AQMS_gbm <- dplyr::full_join(df_all_model_raw_at_AQMS_gbm, read_model_raw_at_AQMS_df)
  } else{
    df_all_model_raw_at_AQMS_gbm <- read_model_raw_at_AQMS_df
  }
  # df_all_model_raw_at_AQMS_gbm  <- rbind.fill(df_all_model_raw_at_AQMS_gbm, read_model_raw_at_AQMS_df)
}



# AQMS interpolated to AQMS as covariate for GBM ###############################

print("Interpolating to AQMS for GBM")

# We pick up the observations configured in config file:
obs_filled       <- df_obs_years 
# AQMS_obs         <- subset(df_obs_years, select = -date)
# replace NANs with NA
AQMS_obs <- df_obs_years %>% mutate_if(is.numeric, ~(ifelse(is.nan(.), NA, .)))

list_gbm_results <- NULL

# To fill the gaps of observations in each station we need a loop to repeat the fill process in each station:
for (s in 1:length(names_local_stations)){
#TODO tryCatch
  
  print(paste0("station ",s," of ",length(names_local_stations)))
  
  station  <- names_local_stations[[s]]
  
 # station is the code station that we hidded to interpolate to it, first
  # col_station           <- which(station==colnames(df_obs_years)) # index de l'estació en les obs i treure-la
  # stations_to_interp    <- df_obs_years[,-col_station] #obs de les altres estacions sense l'estació a interpolar
  
  stations_to_interp <- df_obs_years %>% 
    dplyr::select(-all_of(station))
  
  c <- 0
  OK_other_obs_df  <- NULL    ##reset for each station
  
  #Loop over all days to interpolate date from other station to station
  for(j in 1:length(df_obs_years$date)){
    #We take the data day by day /hour by hour
    ok_obs    <- stations_to_interp[j,] #daily obs for each day of all stations except the hidded one
    ok_obs    <- subset(ok_obs,select=-date)
    df_ok_obs <- data.frame(ref_obs = colnames(ok_obs), pollutant = as.numeric(ok_obs)) 
    
    #coordinates where to interpolate:
    df_hidded_AQMS <- local_stations[ which(local_stations$ref_obs == station) , ] #spatial PDF of station that will be interpolated
    
    if (compute_in_UTM) {
      df_hidded_AQMS <- spTransform(df_hidded_AQMS, sp::CRS(crs_utm))
    }
    
    #    1.2 Compute Variogram
    #with other AQMS we want a SPDF that has the ref of the station and the coordinates of each, with the value of the pollution in that day/hour
    sp_other_ok_obs <- local_stations[ -which(local_stations$ref_obs == station) , ] #spatial points dataframe with all stations except one
    df_other_obs <- as.data.frame(sp_other_ok_obs) #transform from spdf to df
    df_obs_data <- merge(df_other_obs[, -which(names(df_other_obs)=="pollutant")], 
                         df_ok_obs, by = "ref_obs") #get coordinates of stations
    
    if(!sum(is.na(df_obs_data$pollutant))>=length(df_obs_data$pollutant)){
      # df_obs_data <- na.omit(df_obs_data)
      df_obs_data <- df_obs_data %>% 
        dplyr::select(where(~!all(is.na(.x)))) #drop columns with all NA
      
      df_obs_data <- df_obs_data %>%
        dplyr::filter(!is.na(pollutant))
      
      if(file_format=="ghost"){
        coordinates(df_obs_data) <- ~lon_obs+lat_obs
      }else{
        if(colnames(coordinates(local_stations))[1]=="x"){
          coordinates(df_obs_data) <- ~x+y
        }else{
          coordinates(df_obs_data) <- ~lon+lat
        }
      }
      
      sp::proj4string(df_obs_data) <- crs_latlon
      
      if (compute_in_UTM) {
        df_obs_data <- spTransform(df_obs_data, sp::CRS(crs_utm))
      }
      
      if (sum(!is.na(df_obs_data$pollutant)) >= 4){
        tryCatch(
          expr = {
            pollutant.fit_ok <- automap::autofitVariogram(formula=pollutant~1, 
                                                          input_data=df_obs_data,
                                                          model="Ste")
            
            ok_gstat <- gstat(formula=pollutant~1, data = df_obs_data, model = pollutant.fit_ok$var_model, nmax=100)
            OK       <- predict(object=ok_gstat, newdata=df_hidded_AQMS)
            
            OK_other_obs           <- data.frame(data=OK$var1.pred)
            colnames(OK_other_obs) <- station
            OK_other_obs$date      <- stations_to_interp$date[j]
            
            OK_other_obs_df <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
          },
          error = function(e){
            print("!Warning: could not fit variogram")
          }
        )
        
      }else{
        next
      }
    } else{
      #print(paste0("There are no observations in: ", stations_to_interp$date[j]))
      OK_other_obs           <- data.frame(data=NA)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df        <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      c <- c+1
    }
    
  }
  print(paste0("The number of days without any observational data is: ", c))

  # Select station data
  single_station_data <- data.frame(date            = OK_other_obs_df$date,
                                    # obs             = AQMS_obs[,station],
                                    # model           = df_all_model_raw_at_AQMS_gbm[,station], 
                                    day_of_year     = yday(OK_other_obs_df$date),
                                    day_of_week     = wday(OK_other_obs_df$date),
                                    other_obs       = OK_other_obs_df[,station],
                                    year            = year(OK_other_obs_df$date)
                                    )
  
  AQMS_obs_at_station <- AQMS_obs %>%
    dplyr::select(date,all_of(station)) %>%
    dplyr::rename("obs"=any_of(station))
  
  model_at_station <- df_all_model_raw_at_AQMS_gbm %>% 
    dplyr::select(date, contains(station)) %>%
    dplyr::rename("model"=any_of(station)) %>%
    mutate(date = as.Date(date, format="%Y%m%d", tz="UTC"))
  
  single_station_data <- full_join(single_station_data, 
                                   model_at_station, 
                                   by="date")
  
  single_station_data <- full_join(single_station_data,
                                   AQMS_obs_at_station,
                                   by="date")
  
  test_ind      <- which(is.na(single_station_data$obs))
  data_training <- single_station_data[-test_ind,]
  data_test     <- single_station_data[test_ind,]
  
  data_training   <- na.omit(data_training)
  data_test       <- data_test[!is.na(data_test$model),]
  # data_test       <- data_test[!is.na(data_test$other_obs),]
  
  # Preliminary fit

  
  n.trees = 500
  # Vx_formula <- "obs ~ model + day_of_year + day_of_week+ other_obs"
  Vx_formula <- "obs ~ model + day_of_year + day_of_week+ other_obs + year"
  gbm1 <- gbm(as.formula(Vx_formula),
              data = data_training,
              distribution = "gaussian",
              n.trees = n.trees,
              interaction.depth = 1,
              # shrinkage = 0.01,
              cv.folds = 5, #cross validation
              n.cores = 1, # will use all cores by default
              verbose = TRUE)  

  
  # single_influence <- summary(
  #   gbm1,
  #   cBars = 10,
  #   method = relative.influence,  # also can use permutation.test.gbm
  #   las = 2) %>%
  #   dplyr::select(-var) %>%
  #   t.data.frame()   %>%
  #   as.data.frame()
  # 
  # # add the name of the station
  # single_influence$station <- station
  # 
  # # merge with previous influences
  # if (exists("total_influence")) {
  #   total_influence <- dplyr::full_join(total_influence,single_influence)
  # } else{
  #   total_influence <- single_influence
  # }
  
  Test_results_station  <-  data.frame(date=df_obs_years$date) 
  
  res_test     <- predict( gbm1 , newdata=data_test[,c("model","day_of_year","day_of_week","other_obs", "year")])
  
  
  Test_results <- data.frame(date = data_test[,'date'], Test_results=res_test )
  
  
  Test_results_station <- left_join(Test_results_station,Test_results, by="date") %>% 
    distinct()
  
  # Test_results_station[Test_results_station$date %in% Test_results$date ,"Test_results"] <- Test_results[,"Test_results"]
  
  # identify the index of dates with missing observations for this station
  obs_ind      <- which(is.na(df_obs_years[,station]))
  
  # fill the gap with the value we predicted
  df_obs_years[obs_ind,station] <- Test_results_station$Test_results[obs_ind]
}

df_gbm_results <- df_obs_years


if (save_gbm){
  saveRDS(object=df_gbm_results, file=paste0(save_result,"gbm_gap_filling_",file_format,"_", pollutant,"_df.rds"))
  # saveRDS(total_influence, paste0(save_result,"gbm_influences_", pollutant,".rds"))
  print("GBM saved")
} else {
  print("GBM not saved")
}



