
############################################################
#                                                          #
#                          Libraries                       #
#                                                          #
############################################################
# 23-05-2023 PM2,5 de 2022
# Substitue all NA's in df_obs_sub by gbm predictions based on ok_total.

# Needs packages:
library(urbanR)
library(gbm)
library(lubridate)
library(plyr)
library(dplyr) #per treballar amb cbinds i esas cosas
library(caret)


#borrar
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
#library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
#library(dplyr) 
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
library(tidyr)
#library(caret) # an aggregator package for performing many machine learning models

############################################################
#                                                          #
#                        Initiall data                     #
#                                                          #
############################################################
pollutant       <- "pm2p5"
#years           <- c(2020, 2021, 2022)
years           <- 2019
save_result     <-"/esarchive/scratch/ccarnere/RScripts/results_regional_"
file_format     <-"gene"
time_resolution <-"daily"
compute_in_UTM  <- "TRUE"
crs_latlon      <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"

path_gbm    <- "/esarchive/scratch/ccarnere/RScripts/results_regional_2019/GBM_eval/"
############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
#We will pick the latest data available
local_stations     <- readRDS(paste0(save_result,years[length(years)],"/results_gene_",pollutant, "/output/","local_stations_",file_format, "_",pollutant,".rds"))

############################################################
#                                                          #
#                  Load All observations                   #
#                                                          #
############################################################
df_obs_years <- NULL
for(read_year in years){
  df_obs_all        <- readRDS(file = paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","df_obs_",file_format,"_",pollutant,".rds"))
  
  start_date  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date    <- as.Date(paste(read_year, "-12-31", sep = ""))
  
  dates      <- seq(start_date, end_date, by = "day")
  df_obs_sub <- subset(df_obs_all, date>=start_date & date <= end_date)
  
  
  # Comprovation we don't have duplications in obs
  if(any(duplicated(df_obs_sub$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub$date[duplicated(df_obs_sub$date)]))
    df_obs_sub <- distinct(df_obs_sub, date, .keep_all = TRUE)
    print("The df_obs_sub (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub$date) != length(dates)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub)
}

names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_years)))

############################################################
#                                                          #
#          Setting test and training data                  #
#                                                          #
############################################################
# # Create an empty data frame to store non-NA data
# df_not_na <- data.frame(date = as.Date(character(0)), stringsAsFactors = FALSE)
# 
# # Create an empty data frame to store NA data
# df_na     <- data.frame(date = as.Date(character(0)), stringsAsFactors = FALSE)
# 
# for (i in 1:(length(df_obs_years)-1)) {
#   station <- names(df_obs_years)[i]
#   station_data <- df_obs_years[, c("date", station)]
#   
#   # Filtrar los datos no nulos y añadirlos al data frame correspondiente
#   not_na_data <- station_data[!is.na(station_data[, station]), ]
#   df_not_na   <- rbind(df_not_na, not_na_data)
#   
#   # Filtrar los datos nulos y añadirlos al data frame correspondiente
#   na_data <- station_data[is.na(station_data[, station]), ]
#   df_na <- rbind(df_na, na_data)
# }


# 
# 
# 
# # Iterate over each station and separate non-NA and NA data
# for (s in 1:length(names_local_stations)) {
#   stations     <- names_local_stations[[s]]
#   index        <- which(stations==colnames(df_obs_years))
#   station_data <- df_obs_years[,index]
#   
#   # Separate non-NA data and add it to the corresponding data frame
#   non_na_data <- station_data[complete.cases(station_data), ]
#   df_not_na   <- merge(df_not_na, non_na_data, by = "date", all = TRUE)
#   
#   # Separate NA data and add it to the corresponding data frame
#   na_data <- station_data[is.na(station_data[, station]), ]
#   df_na <- merge(df_na, na_data, by = "date", all = TRUE)
# }
# 
# # Print the resulting data frames
# str(df_not_na)
# str(df_na)


############################################################
#                                                          #
#            Load raw model interpolated to AQMS           #
#                                                          #
############################################################
df_all_model_raw_at_AQMS <- NULL
for(read_year in years){
  
  model_raw_at_AQMS_df     <- readRDS(paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  df_all_model_raw_at_AQMS <- rbind.fill(df_all_model_raw_at_AQMS, model_raw_at_AQMS_df)
}

############################################################
#                                                          #
#   AQMS interpolated to AQMS as covariate for GBM         #
#                                                          #
############################################################

# We pick up the observations configured in config file:
obs_filled       <- df_obs_years 
AQMS_obs         <- subset(df_obs_years, select = -date) #we remove dates from observations data frame
OK_other_obs_df  <- NULL
list_gbm_results <- NULL

# To fill the gaps of observations in each station, so we need a loop to repeat the fill process in each station:
for (s in 1:length(names_local_stations)){
  #s <- 1
  station                 <- names_local_stations[[s]]
  
 # Arrangement of AQMS interpolated to AQMS
 # station is the code station that we hidded to interpolate to it, first
  col_station           <- which(station==colnames(df_obs_years)) # index de l'estació en les obs i treure-la
  stations_to_interp    <- df_obs_years[,-col_station] #obs de les altres estacions sense l'estació a interpolar
  c <- 0
  #Loop over all days to interpolate date from other station to station
  for(j in 1:length(df_obs_years$date)){
    #j <- 1
    #print(paste0("Date: ", stations_to_interp$date[j]))
    #We take the data day by day /hour by hour
    ok_obs    <- stations_to_interp[j,] #obs per dia de totes les estacions menys la eliminada
    ok_obs    <- subset(ok_obs,select=-date)
    df_ok_obs <- data.frame(ref_obs = colnames(ok_obs), pollutant = as.numeric(ok_obs)) #data frame estacions x 1 dia
    
    #coordinates where to interpolate:
    df_hidded_AQMS <- local_stations[ which(local_stations$ref_obs == station) , ] #spatial PDF de l'estació a interpolar
  
    if (compute_in_UTM) {
      df_hidded_AQMS <- sp::spTransform(df_hidded_AQMS, sp::CRS(crs_utm))
    }
    
    #    1.2 Compute Variogram
    #with other AQMS we want a SPDF that has the ref of the station and the coordinates of each, with the value of the pollution in that day/hour
    sp_other_ok_obs <- local_stations[ -which(local_stations$ref_obs == station) , ] #spatial PDF on hi han totes les coords excepte l'amagada (sobren estacions)
    df_other_obs <- as.data.frame(sp_other_ok_obs) #pasem de spdf a df per poder filtrar
    df_obs_data  <- merge(df_other_obs[, -which(names(df_other_obs) == "pollutant")], df_ok_obs, by = "ref_obs") #copiem les coordenades de les estacions d'interès amb les observacion que pertoquen
    
    if(!sum(is.na(df_obs_data$pollutant))>=length(df_obs_data$pollutant)){
      df_obs_data                   <- na.omit(df_obs_data)
      coordinates(df_obs_data)      <- ~lon+lat
      sp::proj4string(df_obs_data)  <- crs_latlon
      
      if (compute_in_UTM) {
        df_obs_data <- spTransform(df_obs_data, sp::CRS(crs_utm))
      }
      
      pollutant.fit_ok          <- automap::autofitVariogram(formula=pollutant~1, input_data=df_obs_data,model="Ste")
      
      ok_gstat <- gstat(formula=pollutant~1, data = df_obs_data, model = pollutant.fit_ok$var_model, nmax=100)
      OK       <- predict(object=ok_gstat, newdata=df_hidded_AQMS)
      
      
      OK_other_obs           <- data.frame(data=OK$var1.pred)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      
    } else{
      #print(paste0("There is no observations in: ", stations_to_interp$date[j]))
      OK_other_obs           <- data.frame(data=NA)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df        <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      c <- c+1
    }
    
  }
  print(paste0("The number of days without any observational data is: ", c))
  
  # Select station data
  single_station_data <- data.frame(date            = OK_other_obs_df$date,
                                    obs             = AQMS_obs[,station],
                                    model           = df_all_model_raw_at_AQMS[,station], 
                                    day_of_year     = yday(OK_other_obs_df$date),
                                    day_of_week     = wday(OK_other_obs_df$date),
                                    other_obs       = OK_other_obs_df[,station])
  #estacions interpolat a l'estació
  
  single_station_data                     <- na.omit(single_station_data)
  
  # Split train/test fractions 
  train.fraction  <- 0.90
  train.fraction  <- 0.75
  smp_size        <- floor(train.fraction * nrow(single_station_data)) #te redondea el número i ya
  
  #seq_len et cre una seqüència de números nrow=47 seq_len de l'1 al 47
  #sample agafa un subconjunt aleatòri d'elements d'un vector, en aquest cas agafa el 90% (size)
  set.seed(16) #li he afegit, ns si te sentit que a cada estacio agafi el mateix train_id ---> Preguntar JAN
  train_ind     <- sample(seq_len(nrow(single_station_data)), size = smp_size) #42 valors aleatòris (90% de 48 h)
  data_training <- single_station_data[train_ind,]
  data_test     <- single_station_data[-train_ind,]
  
  # Preliminary fit

  n.trees = 5000 #realment s'ha de provar i escollir l'òptim (si dona temps bucle amb 3-4 opcions per comparar) [la que dongui menor mse]
  Vx_formula <- "obs ~ model + day_of_year + day_of_week+ other_obs"
  gbm1 <- gbm(as.formula(Vx_formula),
              data = data_training,
              distribution = "gaussian",
              n.trees = n.trees,
              interaction.depth = 1, #amb 1 hauria de funcionar, no solem posar més de 10
              shrinkage = 0.01, #learning rate ni molt petit pk anirà lent ni molt gran pk te pasas [ns si bucle amb 3 proves]
              cv.folds = 5, #cross validation
              n.cores = NULL, # will use all cores by default
              verbose = FALSE)  
  gbm.fit <- gbm1
  
  # print results
  print(gbm.fit)
  
  # plot loss function as a result of n trees added to the ensemble
  gbm.perf(gbm.fit, method = "cv")
  
  # get MSE 
  sqrt(min(gbm.fit$cv.error))
  
  # Variable importance
  gbm.fit.final <- gbm.fit
  par(mar = c(5, 8, 1, 1))
  summary(
    gbm.fit.final, 
    cBars = 10,
    method = relative.influence, # also can use permutation.test.gbm
    las = 2)
  
  # Data frame amb dos variables date(fecha + hora) i Tests_results/Obs_Test/Obs_Train (NA)
  Test_results_station  <-  data.frame(date=df_obs_years$date , Test_results = NA) 
  Obs_Test_station      <-  data.frame(date=df_obs_years$date , Obs_Test = NA)
  Obs_Train_station     <-  data.frame(date=df_obs_years$date , Obs_Train = NA)
  
#Once you have decided on a final model you will likely want to use the model to predict on new observations. 
  res_training <- predict( gbm1 , newdata=data_training)
  res_test     <- predict( gbm1 , newdata=data_test[,c("model","day_of_year","day_of_week","other_obs")])
  
  Test_results <- data.frame(date = data_test[,'date'], Test_results=res_test )
  Test_results_station[Test_results_station$date%in%Test_results$date ,"Test_results"] <- Test_results[,"Test_results"]
  
  Obs_Test <- data.frame(date = data_test[,'date'], Obs_Test=data_test$obs )
  Obs_Test_station[Obs_Test_station$date%in%Obs_Test$date ,"Obs_Test"] <- Obs_Test[,"Obs_Test"]
  
  Obs_Train <- data.frame(date = data_training[,'date'], Obs_Train=data_training$obs )
  Obs_Train_station[Obs_Train_station$date%in%Obs_Train$date ,"Obs_Train"] <- Obs_Train[,"Obs_Train"]
  
  data_single     <- data.frame(date                    = df_obs_years$date,
                                Obs_Train_station       = Obs_Train_station$Obs_Train,
                                Obs_Test_station        = Obs_Test_station$Obs_Test,
                                Test_results_station    = Test_results_station$Test_results,
                                model                   = df_all_model_raw_at_AQMS[,station])
  list_gbm_results[[ s ]] <- data_single
  
  # TIME_PLOT
  #png(file = paste0(save_time_series,local_stations$ref_obs[l],"_", local_stations$station_name[l],"_",mm,".png"), width = 800, height = 400)
  #openair::timePlot(data_single,
  #                   pollutant = c("Obs_Train_station","Obs_Test_station","Test_results_station","model"),
  #                   group = TRUE,
  #                   title=station,
  #                   plot.type= "h",
  #                   y.relation = "free",
  #                   ylab="PM2,5 (ug/m3)",
  #                   cols = c("black", "blue", "red", "green"),
  #                   lty = c("dotted","solid","solid","dashed"),
  #                   lwd = c(3,2,2,2), 
  #                   name.pol = c("Obs Train station","Obs Test station", "Test results station", "Model"))
  # dev.off()
  
  #Summary_stats <- openair::modStats(data_single, obs = "Obs_Test_station", mod = "Test_results_station")
  #print(Summary_stats)
  #summary(data_single)
  
  #analisi de la cross validation
  test_performace     <- postResample(pred = res_test, obs = data_test$obs)
  training_performace <- postResample(pred = res_training, obs = data_training$obs)
  
}

saveRDS(object=list_gbm_results, file=paste0(path_gbm,"gbm_results.rds"))
list_gbm_results <- readRDS(file=paste0(path_gbm,"gbm_results.rds"))

# ----------
# #nope, preguntar al jan
# Obs_Test_station          <- matrix(nrow = length(list_gbm_results[[1]]$date), ncol = 0)
# Test_results_station      <- matrix(nrow = length(list_gbm_results[[1]]$date), ncol = 0)
# 
# for (a in 1:length(list_gbm_results)){
#   Obs_Test_station     <- cbind(Obs_Test_station, list_gbm_results[[a]]$Obs_Train_station)
#   Test_results_station <- cbind(Test_results_station, list_gbm_results[[a]]$Test_results_station)
# }
# 
# 
# obs_gbm      <- data.frame( obs = Obs_Test_station, mod=Test_results_station)
# 
# Summary_stats <- openair::modStats(obs_gbm, obs = "obs", mod = "mod",)
# 
# print(Summary_stats)
# 
# 
# 
# 
# # ---------
# Summary_stats <- openair::modStats(list_gbm_results, obs = "Obs_Test_station", mod = "Test_results_station")
# print(Summary_stats)
# 
# df            <- do.call(rbind, list_gbm_results)
# Summary_stats <- openair::modStats(df, obs = "Obs_Test_station", mod = "Test_results_station")

#summary(data_single)

for(i in 1:length(list_gbm_results)){
  data_frame_stats <- as.data.frame(list_gbm_results[i])
  Summary_stats    <- openair::modStats(data_frame_stats, obs = "Obs_Test_station", mod = "Test_results_station")
  print(Summary_stats)
  print("------------------------------------------------------------------------------------------------------------")
}

#-------

# Crear un nuevo data frame vacío
combined_df <- data.frame()

# Concatenar los data frames
combined_df <- do.call(rbind, list_gbm_results)

# Verificar la estructura del nuevo data frame
#str(combined_df)
Summary_stats    <- openair::modStats(data_frame_stats, obs = "Obs_Test_station", mod = "Test_results_station")
print(Summary_stats)

gridExtra::grid.table(Summary_stats)
dev.off()



# df <- data.frame()
# 
# # Recorrer la lista "list_gbm_results"
# for (i in 1:length(list_gbm_results)) {
#     # Obtener el data frame actual
#     current_df <- list_gbm_results[[i]]
# 
#     # Crear un data frame temporal con las columnas deseadas
#     temp_df <- data.frame(Test_results_station = current_df$Test_results_station,
#                           Obs_Test_station = current_df$Obs_Test_station)
# 
#     # Agregar el data frame temporal al data frame principal
#     df <- rbind(df, temp_df)
# }
# 
# # Imprimir la estructura del data frame resultante
# str(df)
# 
# 
# Summary_stats    <- openair::modStats(df, obs = "Obs_Test_station", mod = "Test_results_station")
# print(Summary_stats)
# 
# gridExtra::grid.table(Summary_stats)

# 
# 
# openair::timePlot(data_single,
#                   #pollutant = c("Obs_Train_station","Obs_Test_station","Test_results_station","model"),
#                   pollutant = c("Obs_Test_station","Test_results_station"),
#                   group = TRUE,
#                   title=station,
#                   plot.type= "h",
#                   y.relation = "free",
#                   ylab="PM2,5 (ug/m3)",
#                   cols = c("blue", "red"),
#                   lty = c("solid","solid"),
#                   lwd = c(4,2), 
#                   name.pol = c("Obs Test station", "Test results station"))