start_time_observations <- Sys.time()

# Load Observations ##########################################################
#This script is called from UK_+_LOOCV to set the observations of Catalonia
#we have a doc where we have the reference of the stations that are in catalonia, 
# if we want to change or amplify them we can change the coordinates of the box in set_stations script

if (file_format=="ghost"){
  
  if(time_resolution=="hourly"){
    print(paste0("Setting obs data as hourly mean data.", " ", "Using Ghost obs data"))
    # matrix where we will save the observations:
    obs                  <- matrix(NA,ncol=(nrow(local_stations@data)),nrow=0)
    df_obs_all           <- data.frame(obs)
    dates_all            <- data.frame(matrix(NA,ncol= 1,nrow=0))
    
    #we transform from a matrix to a data frame, with the reference of the station as a name of the col
    colnames(df_obs_all) <- c(local_stations@data$ref_obs)
    colnames(dates_all)  <- c("date")
    # extract the observations of certain year_month
    for (y_month in year_month) {
      netcdf_obs    <- RNetCDF::open.nc(paste0(obs_path, name_pollutant, "_", y_month, ".nc"))
      
      code_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
      # station_name  <- RNetCDF::var.get.nc(netcdf_obs, "station_name")
      obs_all       <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant)
      
      year            <- as.numeric(substr(y_month, 1, 4))
      month           <- as.numeric(substr(y_month, 5, 6))
      num_days        <- length(RNetCDF::var.get.nc(netcdf_obs, "time"))/24
      date_times_loc  <- data_frame_dates( c(year,month,01,00) , c(year,month,num_days,23) )
      dates_all       <- rbind(dates_all,date_times_loc)
      
      pos_local_stations  <- which(code_obs%in%local_stations@data$ref_obs)
      if(length(pos_local_stations)==0){
        print(paste0("No observations found for month ",month))
        next
      }else{
      
        cat_obs             <- obs_all[,pos_local_stations]
        cat_ref             <- code_obs[pos_local_stations]
        #cat_names           <- station_name[pos_local_stations]
        
        obs_month               <- matrix(NA,ncol=length(cat_obs[1,]),nrow=length(cat_obs[,1])) 
        df_obs_month            <- data.frame(obs_month)
        colnames(df_obs_month)  <- paste0(cat_ref)
        df_obs_month[,]         <- cat_obs*obs_constant
        #df_obs_month            <- cbind(date = date_times_loc$date, df_obs_month)
        
        df_obs_all <- rbind.fill(df_obs_all, df_obs_month) #in case there is no data on a certain date, it fills it with NA
      }
      if ( ncol(df_obs_all)!=ncol(df_obs_month) ) {
        print(paste("ERROR: number of monitoring stations is not constant when loading observations.",
                    "Filled with NA in:",y_month))
      }
    }
    
    df_obs_all <- cbind(dates_all,df_obs_all)
    
  }else{
    print("Setting obs data as daily mean data. Using Ghost obs data")
    # matrix where we will save the observations:
    obs                  <- matrix(NA,ncol=(nrow(local_stations@data)),nrow=0)
    df_obs_all           <- data.frame(obs)
    dates_all            <- data.frame(matrix(NA,ncol= 1,nrow=0))
    
    #we transform from a matrix to a data frame, with the reference of the station as a name of the col
    colnames(df_obs_all) <- c(local_stations@data$ref_obs)
    colnames(dates_all)  <- c("date")
    
    for (y_month in year_month) {
      #obs_path<-"/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/observacions/obs_no2"
      netcdf_obs    <- RNetCDF::open.nc(paste0(obs_path, name_pollutant, "_", y_month, ".nc"))
      
      code_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
      #station_name  <- RNetCDF::var.get.nc(netcdf_obs, "station_name")
      time          <- RNetCDF::var.get.nc(netcdf_obs, 'time')
      
      nc_dates      <- sprintf(paste0(y_month,"%02d"),time+1) %>%
                        as.Date(format="%Y%m%d")
      
      obs_all       <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant)
      
      year            <- as.numeric(substr(y_month, 1, 4))
      month           <- as.numeric(substr(y_month, 5, 6))
      num_days        <- length(RNetCDF::var.get.nc(netcdf_obs, "time"))
      #date_times_loc  <- data_frame_dates(date_time_begin=date_time_begin, date_time_end=date_time_end)
      #dates_all       <- rbind(dates_all,date_times_loc)
      
      pos_local_stations  <- which(code_obs%in%local_stations@data$ref_obs)
      cat_obs             <- obs_all[,pos_local_stations]
      cat_ref             <- code_obs[pos_local_stations]
      #cat_names           <- station_name[pos_local_stations]
      
      obs_month               <- matrix(NA,ncol=length(cat_obs[1,]),nrow=length(cat_obs[,1])) 
      df_obs_month            <- data.frame(obs_month)
      colnames(df_obs_month)  <- paste0(cat_ref)
      df_obs_month[,]         <- cat_obs*obs_constant
      #df_obs_month            <- cbind(date = date_times_loc$date, df_obs_month)
      df_obs_month <- cbind(data.frame("date"=nc_dates), df_obs_month)
      
      df_obs_all <- rbind.fill(df_obs_all, df_obs_month) #in case there is no data on a certain date, it fills it with NA
      if ( ncol(df_obs_all)!=ncol(df_obs_month) ) {
        print(paste("ERROR: number of monitoring stations is not constant when loading observations.",
                    "Filled with NA in:",y_month))
      }
    }
  }

}else if (file_format=="gene") {
    if(time_resolution=="hourly") {
    # If the data that we have is hourly (like no2 and o3):
    print(paste0("Setting obs data as hourly mean data.", " ", "Using Gene obs data"))
    
    #read the file and extract the station names, and the dates of the pollutant measures:
    if(!endsWith(obs_path,".csv")){
      input_obs <- read.csv(paste0(obs_path,"gene_",name_pollutant,"_",read_year,".csv"))
    }else{
      input_obs <- read.csv(obs_path)
    }
    
    # sometimes the input files have wrong date formats in which the first hour of the day is
    # shown with format "%Y-%m-%d", and the rest of the hours are in format "%Y-%m-%d %H:%M:%S"
    # fix it, adding hour 00:00:00 to the rows that have the sorter format
    input_obs$Date <- ifelse(grepl("^\\d{4}-\\d{2}-\\d{2}$", input_obs$Date), 
                             paste0(input_obs$Date, " 00:00:00"), 
                             input_obs$Date)
    
    ############################################################
    #           !!!!   WARNING!!!!                             #
    ############################################################
    # Hem posat tz= UTC, cal assegurar-nos que es així
    input_obs$Date <- as.POSIXct(input_obs$Date, format="%Y-%m-%d %H:%M:%S", tz = "UTC")
    input_obs[,-1] <- dplyr::mutate_all(input_obs[,-1],as.numeric)
    
    df_obs_all              <- input_obs
    colnames(df_obs_all)[1] <- "date"
   
    }else if(time_resolution=="daily") {
      
      print("Setting obs data as daily mean data. Using Gene obs data")
      
      #read the file and extract the station names, and the dates of the pollutant measures:
      input_obs      <- read.csv(paste0(obs_path,"gene_",name_pollutant,"_",read_year,".csv"))
      input_obs$Date <- as.Date(input_obs$Date, format = "%Y-%m-%d")
      #in input_obs we had diferent variables, in numeric, logi or int type, just in case we pass them all to a numeric type
      df_obs_all      <- dplyr::mutate_all(input_obs[,-1],as.numeric)
      df_obs_all$Date <- input_obs$Date 
      colnames(df_obs_all)[colnames(df_obs_all) == "Date"] <- "date"
    } else {
      print("wrong time_resolution")
    }
   
}else if (file_format=="xvpca") {
  if(time_resolution=="hourly") {
    print(paste0("Setting obs data as hourly mean data. Using XVPCA obs data"))
    
    obs <- read.csv(paste0(obs_path,"xvpca_",name_pollutant,"_",read_year,".csv"))
    
    # assuming tz= UTC...
    obs$Date <- as.POSIXct(obs$Date, format="%Y-%m-%d %H:%M:%S", tz = "UTC")
    obs[,-1] <- dplyr::mutate_all(obs[,-1],as.numeric)
    
    df_obs_all              <- obs
    colnames(df_obs_all)[1] <- "date"
    
  } else {
    print("** Wrong time_resolution. XVPCA only implemented for hourly data**")
#TODO implement also extracting manual stations
  }
  
} else {
  print("wrong file_format")
}


if (save_obs){
  saveRDS(object=df_obs_all, file=paste0(save_result,"df_obs_",file_format,"_",pollutant,".rds"))
  print("Observations saved")
} else {
  print("Observations have not been saved")
}

end_time_observations       <- Sys.time()
temps_total_observations    <- end_time_observations - start_time_observations
print(paste0("Load observations, execution time: ",temps_total_observations, "s" ))
