library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) 
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
library(tidyr)
#library(RColorBrewer)
#library(Metrics)
library(gbm)
library(lubridate)


library(matrixStats)


pollutants       <- c("pm2p5")
years            <- c(2023) 


pol <- pollutants
year <- years
gbm <- FALSE


general_path <- "/esarchive/scratch/ccarnere/RScripts/results_regional_"
save_sp      <- paste0(general_path,year,"/results_gene_",pol,"/output/")


compute_in_UTM   <- "TRUE"
crs_latlon       <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm          <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"


#my_box            <- rgeos::bbox2SP(43.02355,40.24870,3.497162,-0.001190186, proj4string = CRS(crs_latlon)) # lat màx, lat min, lon max, lon min
my_box            <- rgeos::bbox2SP(43.0,40.25,3.485,0.01, proj4string = CRS(crs_latlon)) # lat màx, lat min, lon max, lon min
if (compute_in_UTM) {
  my_box <- spTransform(my_box, sp::CRS(crs_utm))
}
# Generate an uniform mesh
receptor_grid                     <- sf::st_sf(geom=sf::st_make_grid(my_box, n=c(300,300), what = "centers"), crs=crs_latlon)
sp_receptor_grid                  <- sf::as_Spatial(receptor_grid)
sp::proj4string(sp_receptor_grid) <- crs_utm

#rang <- c(0,10,20,30,40,80,120,160,200,300,400)
rang <-seq(0, 25, 2)
rang_annual <- c(0, 5, 10, 15, 20, 25, 100)

if(gbm){
    filesname <- "UK_gene_pm2p5_gbm" #uk gbm 
}else{
    filesname <- "UK_gene_pm2p5"     #uk
}



p<- 1
pollutant     <- pollutants[p]
save_final_sp <- paste0(save_sp,pollutant,"_matrix")


y <- 1
year            <- years[y]

pollutant_path  <- paste0(general_path,year,"/results_gene_",pollutant, "/output")
#path_model      <- paste0(pollutant_path,"/df_annual_uk.rds")            #ja estic agafant la mitjana
path_model      <- paste0(pollutant_path,"/model_gene_",pollutant,".rds")  
path_coords     <- paste0(pollutant_path,"/coords_model_gene_",pollutant,".rds")

coords_model    <- readRDS(file = path_coords)

lon_model       <- coords_model[, "lon_model"]
lat_model       <- coords_model[, "lat_model"]
#model_corrected <- readRDS(file=path_model)
#df_annual_mean  <- readRDS(file=path_model)


file_list <- list.files(pollutant_path, pattern = paste0(filesname,"_\\d{8}\\.rds$"), full.names = TRUE)
#file_list <- file_list[-6]


sum_matrix <- NULL                    #matriu on posarem la suma total
count_matrix <- numeric(82844)        #matriu de cada cop q se suma valor a la posicio (per dividir quan fem la mitjana)
count_daylimit <- numeric(82844)      #matriu de les vegades q se supera el limit diari



for (file in file_list) {
  #file <- file_list[2]
  current_matrix <- readRDS(file)[[1]]
  current_matrix <- ifelse(current_matrix < 0, 0, current_matrix)  #fer 0 totes les concentracions q donin negatives
  
  #iniciar quan es nul.la
  if (is.null(sum_matrix)) {
    sum_matrix     <- current_matrix
    
    for (i in 1:length(current_matrix)) {
      if (!is.na(current_matrix[i])) {  #mirar si el valor no es nan
      count_matrix[i] <- count_matrix[i] + 1
      }
    }
  } else{
    #sumar sempre q no sigui nan
    for (i in 1:length(current_matrix)) {
      if (!is.na(current_matrix[i])) { 
        sum_matrix[i] <- sum_matrix[i] + current_matrix[i]
        count_matrix[i] <- count_matrix[i] + 1
      }
    }
  }
  
  #comptador diari per veure si supera el limit diari de 15um/m3
  for (i in 1:length(current_matrix)) {
    if (!is.na(current_matrix[i])){
      if (current_matrix[i] > 25) {  
        count_daylimit[i] <- count_daylimit[i] + 1
      }
    }
  }

}

annual_mean <- sum_matrix/count_matrix   


#save the values from each file
#data_list <- lapply(file_list, function(file_path) {
  #data <- readRDS(file_path)
  #return(data[[1]]) 
#})

#data_matrix <- do.call(cbind, data_list)  #convert into a dataframe (rows are points, cols are days)


#Annual mean
#annual_mean                      <- rowMeans(data_matrix, na.rm = TRUE)
df_annual_mean                   <- data.frame(pollutant=annual_mean, lon=lon_model, lat=lat_model)
coordinates(df_annual_mean)      <- ~lon+lat
sp::proj4string(df_annual_mean)  <- crs_latlon

if (compute_in_UTM) {
  df_annual_mean <- spTransform(df_annual_mean, sp::CRS(crs_utm))
}

set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
df_annual_mean     <- na.omit(df_annual_mean)
pick_random_points <- sample(1:nrow(df_annual_mean),30000)
pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_annual_mean[pick_random_points,],model="Ste")
#pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_annual_mean,model="Ste")
ok_gstat           <- gstat(formula=pollutant~1, data = df_annual_mean, model = pollutant.fit_ok$var_model, nmax=25)
OK                 <- predict(object=ok_gstat, newdata=sp_receptor_grid)

#saveRDS(object=OK, file=paste0(general_path,"OK_UK_sp.rds"))

pixels      <- sp::SpatialPixelsDataFrame(OK, as.data.frame(OK[,"var1.pred"]))
#pixels      <- sp::SpatialPixelsDataFrame(OK,tolerance = 0.99, as.data.frame(OK@data$var1.pred))
mean_raster <- raster::raster(pixels[,'var1.pred'])

if(gbm){
    saveRDS(object=pixels, file=paste0(save_sp,"annual_pixels_UK_gbm.rds"))
}else{
    saveRDS(object=pixels, file=paste0(save_sp,"annual_pixels_UK.rds"))
}


#saveRDS(object=mean_raster, file=paste0(save_sp,"annual_mean_raster_UK.rds"))
#saveRDS(object=mean_raster, file=paste0(save_sp,"annual_mean_raster_UK_gbm.rds"))

#pixels_UKgbm <- pixels


rang_annual <- seq(0, 25, 5)
rang <-seq(0, 25)
# mapview::mapview(OK$var1.pred, at=rang)
# mapview::mapview(datos_leidos, at=rang)
#mapview::mapview(mean_raster)
#mapview::mapview(pixels_UKgbm, zcol="var1.pred", at=rang_annual)
#mapview::mapview(pixels_UK, zcol="var1.pred", method="bilinear")
mapview::mapview(pixels, zcol="var1.pred")
#mapview::mapview(pixels, zcol="var1.pred")  #col.regions=viridis(256)


#mapview::mapview(pisels_UK, zcol="var1.pred")
#mapview::mapview(pisels_UKgbm, zcol="var1.pred")




#comptador diari ---------------------------------------------

df_count_daylimit                   <- data.frame(pollutant=count_daylimit, lon=lon_model, lat=lat_model)
coordinates(df_count_daylimit)      <- ~lon+lat
sp::proj4string(df_count_daylimit)  <- crs_latlon

if (compute_in_UTM) {
  df_count_daylimit <- spTransform(df_count_daylimit, sp::CRS(crs_utm))
}

set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
df_count_daylimit     <- na.omit(df_count_daylimit)
pick_random_points <- sample(1:nrow(df_count_daylimit),30000)
pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_count_daylimit[pick_random_points,],model="Ste")
#pollutant.fit_ok   <- automap::autofitVariogram(formula=pollutant~1, input_data=df_annual_mean,model="Ste")
ok_gstat           <- gstat(formula=pollutant~1, data = df_count_daylimit, model = pollutant.fit_ok$var_model, nmax=25)
OK                 <- predict(object=ok_gstat, newdata=sp_receptor_grid)

#saveRDS(object=OK, file=paste0(general_path,"OK_UK_sp.rds"))

pixels      <- sp::SpatialPixelsDataFrame(OK, as.data.frame(OK[,"var1.pred"]))
#pixels      <- sp::SpatialPixelsDataFrame(OK,tolerance = 0.99, as.data.frame(OK@data$var1.pred))
mean_raster <- raster::raster(pixels[,'var1.pred'])
#saveRDS(object=pixels, file=paste0(save_sp,"annual_pixels_count_daylimit_UK_GBM.rds"))
#saveRDS(object=mean_raster, file=paste0(save_sp,"annual_mean_raster_count_daylimit_UK_GBM.rds"))





# mapview::mapview(OK$var1.pred, at=rang)
# mapview::mapview(datos_leidos, at=rang)
#mapview::mapview(mean_raster, at=c(0,2,4, 6, 8, 10, 15, 20, 25, 30, 365))
#mapview::mapview(pixels, zcol="var1.pred", at=rang_annual)
mapview::mapview(pixels, zcol="var1.pred", at=seq(-10,90, 10))
mapview::mapview(pixels, zcol="var1.pred", at=c(0,2,4, 6, 8, 10, 15, 20, 25, 30, 365))
