start_time_stations <- Sys.time()

# Set stations ############################################################
# study area boundaries:
my_box          <-
  rgeos::bbox2SP(lat_domain[1],
                 lat_domain[2],
                 lon_domain[1],
                 lon_domain[2],
                 proj4string = CRS(crs_latlon)) # lat max, lat min, lon max, lon min


if (file_format == "ghost") {
  all_stations = NULL
  for (i in 1:length(year_month)) {
    obs_path_stations <- paste0(obs_path, name_pollutant, "_", year_month[i], ".nc")
    
    #observations:
    netcdf_obs        <- RNetCDF::open.nc(obs_path_stations)
    
    lat_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'latitude')
    lon_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'longitude')
    ref_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
    station_name <- RNetCDF::var.get.nc(netcdf_obs, 'station_name')
    obs_all      <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant) 
    
    df_obs       <-
      data.frame(
        pollutant = NA,
        lat_obs = lat_obs,
        lon_obs = lon_obs,
        ref_obs = ref_obs,
        station_name = station_name
      )
    all_stations <- rbind(all_stations, df_obs)
  }
  
  #remove the duplicates:
  df2               <- all_stations[!duplicated(all_stations$ref_obs),]
  coordinates(df2)  <- ~ lon_obs + lat_obs
  
  sp::proj4string(df2)   <- crs_latlon

  #subset stations within the selected domain
  local_stations <- raster::intersect(df2, my_box) 

  
} else if (file_format == "gene") {
  all_stations = NULL
  for (n in 1:length(read_year)) {
    # df_local_stations           <- read.csv(paste0(stations_path,"XVPCA_info_",name_pollutant,"_","2023",".csv"))
    df_local_stations           <- read.csv(stations_path)
    df_local_stations$pollutant <- NA
    names(df_local_stations)[1] <- "ref_obs"
    names(df_local_stations)[4] <- "type"
    # df_local_stations           <- df_local_stations %>%
    #   dplyr::rename(ref_obs = code) # rename(new_name=old_name)
    df_local_stations$ref_obs   <- as.character(df_local_stations$ref_obs)
    df_local_stations$type      <- as.character(df_local_stations$type)
    
    # lat_obs      <- df_local_stations$lat
    # lon_obs      <- df_local_stations$lon
    # ref_obs      <- df_local_stations$code
    # station_type <- df_local_stations$type
    
    # df_obs       <-
    #   data.frame(
    #     pollutant = NA,
    #     lat_obs = lat_obs,
    #     lon_obs = lon_obs,
    #     ref_obs = ref_obs,
    #     station_type = station_type
    #   )
    # all_stations <- rbind(all_stations, df_obs)
    
    
    all_stations       <- rbind(all_stations, df_local_stations)
  
  }
  
  df2                   <- all_stations[!duplicated(all_stations),]
  coordinates(df2)      <- ~ lon + lat
  sp::proj4string(df2)  <- crs_latlon
  
  # Selecting local stations within domain 
  local_stations <- raster::intersect(df2, my_box) #Fa una llista de les obs de NO2 que estiguin dindre de l'àrea del box

  
} else {
  print("wrong obs_dataset")
}


# save data
if (save_stations) {
  saveRDS(object = local_stations,file = paste0(save_result,"local_stations_",file_format,"_",pollutant,".rds"))
  print("Local stations saved")
} else {
  print("Local stations have not been saved")
}


end_time_stations    <- Sys.time()
temps_total_stations <- end_time_stations - start_time_stations
print(paste0("local stations, execution time:",temps_total_stations, "s" ))
