############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# it alerts us if the dates in model do not coincide with the dates of the obs

# if ( any(date_times$date != df_obs_sub$date))  {
#   print("Warning: dates from model and observations are different")
# }
# no se si treure'l o bé, posar un if segons la time_resolution

#-------------------------------------------------------------------------------
# We will generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))

hourly_pollutants <- c("no2", "o3", "so2", "co", "no")
daily_pollutants  <- c("pm10", "pm2p5")

#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total         <- NULL
UK_model_total          <- NULL 
geostats_results        <- NULL

for (station_code in names_local_stations) {
  # station_code <- names_local_stations[[1]]
  # obs_station       <- df_obs_sub[ ,c("date",station_code)]
  
  # filter obs_station for dates in model_raw_at_AQMS_df
  obs_station <- df_obs_sub %>%
    dplyr::filter(date %in% as.Date(model_raw_at_AQMS_df$date, format="%Y%m%d"))
  obs_station <- obs_station[,station_code]
  
  raw_model_estacio <- data.frame(obs = obs_station,
                                  mod = model_raw_at_AQMS_df[ ,station_code], 
                                  model="raw_model")
  UK_model_estacio <-   data.frame(obs = obs_station,
                                   mod = LOOCV_total_df[ ,station_code],
                                   model="UK_model")
  
  # Statistics for a single station
  Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
  UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
  
  geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                 y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                 COE_raw = Raw_geostats$COE,
                                 COE_UK = UK_geostats$COE,
                                 MB_raw = Raw_geostats$MB,
                                 MB_UK = UK_geostats$MB)
  
  # Gather all stations
  geostats_results    <- rbind(geostats_results,geostats_station)
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
  UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      UK_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()
