############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# it alerts us if the dates in model do not coincide with the dates of the obs

# if ( any(date_times$date != df_obs_sub$date))  {
#   print("Warning: dates from model and observations are different")
# }
# no se si treure'l o bé, posar un if segons la time_resolution

#-------------------------------------------------------------------------------
# We will generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
# data frame with gbm data

#LOOCV_total_gbm_df      <- readRDS(paste0(save_result, "LOOCV_", file_format, "_", pollutant, if (execute_uk_gbm) "_gbm_" else "_", "df.rds"))
#LOOCV_total_gbm_df$date <- df_obs_sub$date

#-------------------------------------------------------------------------------
#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total         <- NULL
UK_model_total          <- NULL 
GBM_model_total         <- NULL 
geostats_results        <- NULL

for (station_code in names_local_stations) {
  # station_code <- names_local_stations[[18]]
  obs_station       <- df_obs_sub[ ,station_code]
  raw_model_estacio <- data.frame(obs = obs_station,
                                  mod = model_raw_at_AQMS_df[ ,station_code], 
                                  model = "raw_model")
  
  UK_model_estacio  <-   data.frame(obs = obs_station,
                                    mod = LOOCV_total_df[ ,station_code],
                                    model = "UK_model")

  
  GBM_model_estacio <-   data.frame(obs = obs_station,
                                    mod = LOOCV_total_gbm_df[,station_code],
                                    model = "GBM_model")
  
  # Statistics for a single station
  Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
  UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
  GBM_geostats    <- openair::modStats(GBM_model_estacio, obs = "obs", mod = "mod", type = "model")
  
  geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                 y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                 ref = station_code,
                                 COE_raw = Raw_geostats$COE,
                                 COE_UK = UK_geostats$COE,
                                 COE_GBM = GBM_geostats$COE,
                                 MB_raw = Raw_geostats$MB,
                                 MB_UK = UK_geostats$MB,
                                 MB_GBM = GBM_geostats$MB)
  
  # Gather all stations
  geostats_results    <- rbind(geostats_results,geostats_station)
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
  UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
  GBM_model_total  <- rbind(GBM_model_total,GBM_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      UK_model_total,
                      GBM_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_GBM_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic for all stations concateneited: ")

##
## Map Statistics
##

geostats_results <- na.omit(geostats_results)

coordinates(geostats_results)        <- ~ x + y
sp::proj4string(geostats_results)    <- crs_latlon

a <- mapview::mapview(geostats_results, zcol = "COE_UK", legend = TRUE,at=c(-10,-0.5,0,0.5,1))
b <- mapview::mapview(geostats_results, zcol = "COE_GBM", legend = TRUE,at=c(-30,-5,-1,-0.5,0,0.5,1))

c <- mapview::mapview(geostats_results, zcol = "MB_UK", legend = TRUE,at=c(-35,-12,-10,-5,0,5,10,12,20,45))
d <- mapview::mapview(geostats_results, zcol = "MB_GBM", legend = TRUE,at=c(-35,-30,-15,-10,-5,0,5,10,15,20))


#to save it in htm:
mapshot(a,paste0(save_result,"COE_UK.html"))
mapshot(b,paste0(save_result,"COE_GBM.html"))


mapshot(c,paste0(save_result,"MB_UK.html"))
mapshot(d,paste0(save_result,"MB_GBM.html"))



#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean            <- matrix(nrow = length_loop, ncol = 0) 
raw_model_total_mean      <- matrix(nrow = length_loop, ncol = 0)
UK_model_total_mean       <- matrix(nrow = length_loop, ncol = 0)
GBM_model_total_mean      <- matrix(nrow = length_loop, ncol = 0)

for (station_code in names_local_stations) {
  # station_code <- local_stations@data$code[1]
  
  obs_total_mean          <- cbind(obs_total_mean,df_obs_sub[ ,station_code])
  raw_model_total_mean    <- cbind(raw_model_total_mean,model_raw_at_AQMS_df[ ,station_code])
  UK_model_total_mean     <- cbind(UK_model_total_mean,LOOCV_total_df[ ,station_code])
  GBM_model_total_mean    <- cbind(GBM_model_total_mean,LOOCV_total_gbm_df[,station_code])
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                                   model="raw_model")

UK_model_mean     <- data.frame(   obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(UK_model_total_mean,na.rm=TRUE),
                                   model="UK_model")

GBM_model_mean     <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                     mod = rowMeans(GBM_model_total_mean,na.rm=TRUE),
                                     model="GBM_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                             UK_model_mean,
                             GBM_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly_",pollutant,"gbm_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic considering the mean hour for all stations: ")

############################################################
#                                                          #
#              Temporal variations                         #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per la mitjana de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total          <- matrix(nrow = length_loop, ncol = 0) 
raw_model_total    <- matrix(nrow = length_loop, ncol = 0)
UK_model_total     <- matrix(nrow = length_loop, ncol = 0)
GBM_model_total    <- matrix(nrow = length_loop, ncol = 0)

for (station_code in names_local_stations) {

  obs_total          <- cbind(obs_total, df_obs_sub[ ,station_code])
  raw_model_total    <- cbind(raw_model_total,model_raw_at_AQMS_df[ ,station_code])
  UK_model_total     <- cbind(UK_model_total,LOOCV_total_df[ ,station_code])
  GBM_model_total <- cbind(GBM_model_total,LOOCV_total_gbm_df[,station_code])
}

#Then, we calculate the means of each hour considering the whole stations
raw_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

UK_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(UK_model_total,na.rm=TRUE),
                          model="corr_model")

GBM_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                              mod = rowMeans(GBM_model_total,na.rm=TRUE),
                              model="corr_model")


Mean_all_stations            <- data.frame(date               = df_obs_sub$date,
                                           observations       = raw_model$obs,
                                           raw_Model          = raw_model$mod, 
                                           UK_Model           = UK_model$mod,
                                           GBM_Model       = GBM_model$mod)

TIME_VAR <- openair::timeVariation(Mean_all_stations,
                                   pollutant = c("observations","raw_Model","UK_Model","GBM_Model"),
                                   ylab = paste0(pollutant, " ", "(ug/m3)"),
                                   type ="month",
                                   key = TRUE,
                                   title = "Media horaria Cat",
                                   auto.text=FALSE,
                                   ci = FALSE,
                                   #conf.int = c(0.75, 0.99),
                                   ylim = c(0, 100),
                                   cols = c("black", "grey", "green","brown"),
                                   lty = c("solid","solid","solid","solid"),
                                   lwd = c(8,2,2,2), 
                                   name.pol = c("Observations", "Raw model", "UK Model","GBM Model"))  

png(file = paste0(save_general_statistics,"temporal_",pollutant,"_gbm_mean_time_VAR.png"), width = 800, height = 600)
print(TIME_VAR[[1]][3])  # per pm! [1][2] no2 i o3
dev.off()
#-------------------------------------------------------------------------------
# Plot per cada estació:


for (station_code in names_local_stations) {
  
  Single_station            <- data.frame(date               = df_obs_sub$date,
                                          observations       = df_obs_sub[ ,station_code],
                                          raw_Model          = model_raw_at_AQMS_df[ ,station_code], 
                                          UK_corrected       = LOOCV_total_df[ ,station_code],
                                          GBM_corrected      = LOOCV_total_gbm_df[,station_code])
  
  TIME_VAR <- openair::timeVariation(Single_station,
                                     pollutant = c("observations","raw_Model","UK_corrected","GBM_corrected"),
                                     ylab = paste0(pollutant, " ", "(ug/m3)"),
                                     type ="month",
                                     key = TRUE,
                                     title = station_code,
                                     auto.text=FALSE, 
                                     ci = FALSE,
                                     conf.int = c(0.75, 0.99),
                                     ylim = c(0, 100),
                                     cols = c("black", "grey", "green","brown"),
                                     lty = c("solid","solid","solid","solid"),
                                     lwd = c(8,2,2,2), 
                                     name.pol = c("Observations", "Raw model", "UK Model","GBM Model"))  
  png(file = paste0(save_temporal_variation,station_code,"_",pollutant,"_gbm_time_VAR.png"), width = 800, height = 600)
  print(TIME_VAR[[1]][3]) #same 
  dev.off()
  
}



TIME_VAR <- openair::timeVariation(Single_station,
                                   pollutant = c("observations","raw_Model","GBM_corrected"),
                                   ylab = paste0(pollutant, " ", "(ug/m3)"),
                                   type ="month",
                                   key = TRUE,
                                   title = station_code,
                                   auto.text=FALSE, 
                                   ci = FALSE,
                                   conf.int = c(0.75, 0.99),
                                   ylim = c(0, 100),
                                   cols = c("black", "grey", "green"),
                                   lty = c("solid","solid","solid"),
                                   lwd = c(8,2,2), 
                                   name.pol = c("Observations", "Raw model","GBM Model"))  
print(TIME_VAR[[1]][2]) #same 
############################################################
#                                                          #
#                   Time series                            #
#                                                          #
############################################################

# Plot per cada estació:
model_raw_at_AQMS=model_raw_at_AQMS_df

#month_names = c("August")
for (station_code in names_local_stations) {
  
  # station_code <- names_local_stations[[1]]
  data_single     <- data.frame(date               = df_obs_sub$date,
                                observations       = df_obs_sub[ ,station_code],
                                Raw_Model          = model_raw_at_AQMS_df[ ,station_code], 
                                UK_Model           = LOOCV_total_df[ ,station_code],
                                GBM_Model       = LOOCV_total_gbm_df[,station_code])
  
  for (mm in month_names) { #no2, o3 per hora o dia separar (pm treure for)
    # TIME_PLOT
    png(file = paste0(save_time_series,station_code,"_",pollutant,"_gbm_",mm,".png"), width = 800, height = 400)
    openair::timePlot(data_single, #openair::selectByDate(data_single, year=read_year, month = mm),
                      pollutant = c("observations","Raw_Model","UK_Model","GBM_Model"),
                      group = TRUE,                                    
                      title = station_code,
                      auto.text=FALSE, 
                      #plot.type = "l", #for o3 and no2
                      ci = FALSE,
                      plot.type = "h", #for pm
                      conf.int = c(0.75, 0.99),
                      ylim = c(0, 150),
                      cols = c("black", "blue", "grey","red"),
                      lty = c("solid","solid","solid","solid"),
                      lwd = c(5,6,3,2),
                      name.pol = c("Observations", "Raw model", "UK Model","GBM Model"))
    dev.off()
  }
}

# #només era per plotat UK & GBM per una estació det
# for (mm in month_names) { #no2, o3 per hora o dia separar (pm treure for)
#   # TIME_PLOT
#   openair::timePlot(data_single, #openair::selectByDate(data_single, year=read_year, month = mm),
#                     pollutant = c("observations","UK_Model","GBM_Model"),
#                     group = TRUE,
#                     title = station_code,
#                     auto.text=FALSE,
#                     plot.type = "l",
#                     ci = FALSE,
#                     #plot.type = "h",
#                     conf.int = c(0.75, 0.99),
#                     ylim = c(0, 200),
#                     cols = c("black", "blue","red"),
#                     lty = c("solid","solid","solid"),
#                     lwd = c(5,3,2),
#                     name.pol = c("Observations", "UK Model","GBM Model"))
#   dev.off()
# }





# UK
for (station_code in names_local_stations) {
  
  # station_code <- names_local_stations[[1]]
  data_single     <- data.frame(date               = df_obs_sub$date,
                                observations       = df_obs_sub[ ,station_code],
                                Raw_Model          = model_raw_at_AQMS_df[ ,station_code], 
                                UK_Model           = LOOCV_total_df[ ,station_code])
  
  for (mm in month_names) { #no2, o3 per hora o dia separar (pm treure for)
    # TIME_PLOT
    png(file = paste0(save_time_series,station_code,"_",pollutant,"_uk_",mm,"_gbm.png"), width = 800, height = 400)
    openair::timePlot(data_single, #openair::selectByDate(data_single, year=read_year, month = mm),
                      pollutant = c("observations","Raw_Model","UK_Model"),
                      group = TRUE,                                    
                      title = station_code,
                      auto.text=FALSE, 
                      plot.type = "h",
                      ci = FALSE,
                      #plot.type = "h",
                      conf.int = c(0.75, 0.99),
                      ylim = c(0, 150),
                      cols = c("black", "blue", "grey"),
                      lty = c("solid","solid","solid"),
                      lwd = c(5,6,2),
                      name.pol = c("Observations", "Raw model", "UK Model"))
    dev.off()
  }
}


for (station_code in names_local_stations) {
  
  # station_code <- names_local_stations[[1]]
  data_single     <- data.frame(date               = df_obs_sub$date,
                                observations       = df_obs_sub[ ,station_code],
                                Raw_Model          = model_raw_at_AQMS_df[ ,station_code], 
                                UK_Model           = LOOCV_total_df[ ,station_code],
                                GBM_Model          = LOOCV_total_gbm_df[,station_code])
  
  for (mm in month_names) { #no2, o3 per hora o dia separar (pm treure for)
    # TIME_PLOT
    png(file = paste0(save_time_series,station_code,"_",pollutant,"_gbm_",mm,".png"), width = 800, height = 400)
    openair::timePlot(data_single, #openair::selectByDate(data_single, year=read_year, month = mm),
                      pollutant = c("observations","Raw_Model","GBM_Model"),
                      group = TRUE,                                    
                      title = station_code,
                      auto.text=FALSE, 
                      plot.type = "h",
                      ci = FALSE,
                      #plot.type = "h",
                      conf.int = c(0.75, 0.99),
                      ylim = c(0, 150),
                      cols = c("black", "blue", "grey"),
                      lty = c("solid","solid","solid"),
                      lwd = c(5,6,2),
                      name.pol = c("Observations", "Raw model","GBM Model"))
    dev.off()
  }
}
#-------------------------------------------------------------------------------
# Plot per la mitjana horària de totes les estacions:
#First, we create a matrix to put the data of observations and model (corrected and raw)

obs_total            <- matrix(nrow = length_loop, ncol = 0) 
raw_model_total      <- matrix(nrow = length_loop, ncol = 0)
corr_model_total     <- matrix(nrow = length_loop, ncol = 0)
GBM_model_total   <- matrix(nrow = length_loop, ncol = 0)

for (station_code in names_local_stations) {

  obs_estacio           <-  df_obs_sub[ ,station_code]
  raw_model_estacio     <-  model_raw_at_AQMS_df[ ,station_code]
  corr_model_estacio    <-  LOOCV_total_df[ ,station_code]
  
  obs_total             <- cbind(obs_total,obs_estacio)
  raw_model_total       <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total      <- cbind(corr_model_total,corr_model_estacio) 
  GBM_model_total    <- cbind(GBM_model_total,LOOCV_total_gbm_df[,station_code])
  
}

#Then, we calculate the means of each hour considering the whole stations
raw_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                           mod = rowMeans(raw_model_total,na.rm=TRUE),
                           model="raw_model")

corr_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                            mod = rowMeans(corr_model_total,na.rm=TRUE),
                            model="corr_model")

GBM_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                              mod = rowMeans(GBM_model_total,na.rm=TRUE),
                              model="corr_model")


data_single   <- data.frame(date               = df_obs_sub$date,
                            observations       = raw_model$obs,
                            Model              = raw_model$mod,
                            Model_corrected    = corr_model$mod,
                            GBM_Model          = GBM_model$mod)


png(file = paste0(save_general_statistics,"time_series_","hourly_mean","_",pollutant,"_gbm.png"), width = 800, height = 400)
openair::timePlot(data_single,
                  pollutant = c("observations","Model","Model_corrected","GBM_Model"),
                  plot.type = "l",
                  group = TRUE,
                  y.relation = "free",
                  ylab= paste0(pollutant, " ", "(ug/m3)"),
                  cols = c("black", "blue", "green","red"),
                  lty = c("solid","solid","solid","solid"),
                  lwd = c(3,3,2,2),
                  name.pol = c("observations","Raw model", "UK Model", "GBM model"))
dev.off()




# 
# openair::timePlot(data_single,
#                   pollutant = c("observations","Model_corrected","GBM_Model"),
#                   plot.type = "l",
#                   group = TRUE,
#                   y.relation = "free",
#                   ylab= paste0(pollutant, " ", "(ug/m3)"),
#                   cols = c("black", "blue", "green"),
#                   lty = c("solid","solid","solid"),
#                   lwd = c(3,3,2),
#                   name.pol = c("observations", "UK Model", "GBM model"))
