# CALCULATION OF STATISTICS
# averaging all stations

# inputs:
#   local_stations
#   df_obs_sub
#   model_raw_at_AQMS_df
#   LOOCV_total_df

# outputs:
#   Summary_stats (png+RDS)
#   Summary_stats_h (png+RDS)
#   COE and MB mapshots


library(mapview)
library(gridExtra)


# generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))

#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparison using all stations concatenated:
raw_model_total         <- NULL
UK_model_total          <- NULL 
geostats_results        <- NULL

for (station_code in names_local_stations) {
  # station_code <- names_local_stations[[1]]
  obs_station       <- df_obs_sub[ ,station_code]
  raw_model_estacio <- data.frame(obs = obs_station,
                                  mod = model_raw_at_AQMS_df[ ,station_code], 
                                  model="raw_model")
  UK_model_estacio <-   data.frame(obs = obs_station,
                                   mod = LOOCV_total_df[ ,station_code],
                                   model="UK_model")
  
  # Statistics for a single station
  Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
  UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
  
  geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                 y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                 COE_raw = Raw_geostats$COE,
                                 COE_UK = UK_geostats$COE,
                                 MB_raw = Raw_geostats$MB,
                                 MB_UK = UK_geostats$MB)
  
  # Gather all stations
  geostats_results    <- rbind(geostats_results,geostats_station)
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
  UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      UK_model_total)

saveRDS(mod_obs_data,
        paste0(save_general_statistics,"mod_obs_stations_",pollutant,"_",year,".RDS"))

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
#TODO CHECK add column pollutant and year
Summary_stats <- Summary_stats %>%
  bind_cols(as.data.frame("pol" = pollutant, 
                          "year"=year,
                          "aggreg"="annual"), 
            .)

saveRDS(Summary_stats,
        paste0(save_general_statistics,"Stats_",pollutant,"_",year,".RDS"))

print(Summary_stats)
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()


# plot model vs obs ----
#TODO import limits

ggplot()


# Generate Maps with COE and MB ----

geostats_results <- na.omit(geostats_results)
coordinates(geostats_results)        <- ~ x + y
sp::proj4string(geostats_results)    <- crs_latlon

coe_uk <- mapview::mapview(geostats_results, zcol = "COE_UK", legend = TRUE,at=c(-30,-5,-1,-0.5,0,0.5,1))
mb_uk  <- mapview::mapview(geostats_results, zcol = "MB_UK", legend = TRUE,at=c(-35,-30,-15,-10,-5,0,5,10,15,20))

#to save it in html:
mapshot(coe_uk,paste0(save_result,"COE_UK.html"))
mapshot(mb_uk,paste0(save_result,"MB_UK.html"))

print("Statistics for all stations: ")




# Mean value for each hour all stations -----
obs_total_mean            <- matrix(nrow = length(df_obs_sub$date), ncol = 0) 
raw_model_total_mean      <- matrix(nrow = length(df_obs_sub$date), ncol = 0)
UK_model_total_mean       <- matrix(nrow = length(df_obs_sub$date), ncol = 0)

for (station_code in names_local_stations) {
  obs_station <- df_obs_sub[ ,station_code]
  # station_code <- local_stations@data$code[1]
  
  raw_model_estacio  <-  model_raw_at_AQMS_df[ ,station_code]
  UK_model_estacio   <-  LOOCV_total_df[ ,station_code]
  
  obs_total_mean          <- cbind(obs_total_mean,obs_station)
  raw_model_total_mean    <- cbind(raw_model_total_mean,raw_model_estacio)
  UK_model_total_mean     <- cbind(UK_model_total_mean,UK_model_estacio)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                                   model="raw_model")

UK_model_mean     <- data.frame(   obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(UK_model_total_mean,na.rm=TRUE),
                                   model="UK_model")


mod_obs_data_h <- rbind(raw_model_mean,
                      UK_model_mean)

saveRDS(mod_obs_data_h,
        paste0(save_general_statistics,"mod_obs_stations_hourly_",pollutant,"_",year,".RDS"))

Summary_stats_h <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")

#TODO CHECK add column pollutant and year
Summary_stats_h <- Summary_stats_h %>%
  bind_cols(as.data.frame("pol" = pollutant, 
                          "year"=year,
                          "aggreg"="hourly"), 
            .)

saveRDS(Summary_stats_h,
        paste0(save_general_statistics,"Stats_hourly_",pollutant,"_",year,".RDS"))

print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()
