# script original M.T.Pay per fer els plots de la Diagnosi de bias per estació
# llegeix un fitxer csv que conté els camps:
# CODI;LAT;LON;PM10_MEAN_OBS;PM10_MEAN_EXP;PM10_PERC_OBS;PM10_PERC_EXP
# exemple csv: /esarchive/scratch/molid/caliope_cat_results_2023/cat/results_csv/resultats_pm10_2023.csv

#TODO adaptar per UK

rm(list=ls())

library(reshape2)
library(readxl)
library(dplyr)
library(stringr)
library(ggplot2)
library(grid)
library (gridExtra)


############ USER OPTIONS ##################
####### O3
#pol<-c("O3")
#tb<-c("PERC","8HMAX")
####### NO2
#pol<-c("NO2")
#tb<-c("PERC","MEAN")
####### PM10
pol<-c("PM10")
tb<-c("PERC","MEAN")

year<-c(2019)
expid<-c("b083")
path_out<-"/esarchive/scratch/molid/CALIOPE_CAT_results/results_csv/plots2/"
path_in<-"/esarchive/scratch/molid/CALIOPE_CAT_results/results_csv/"

############ END USER OPTIONS ##################

## READING METADATA
#df<-read.csv("/esarchive/scratch/mtpay/output_CALIOPE_CAT/data/CMAQ-IP4-nearestneighbour-EIONET.csv",skip=1,header=F,sep=" ")
#colnames(df)<-c("ix","iy","station")
#df$id<-as.factor(paste0(df$ix,"_",df$iy))

## add station name:
#path.in<-"/shared/earth/AtmosphericComposition/CALIOPE_CAT/Dades_XVPCA_2015-2018/"
path.in<-"/esarchive/oper/CALIOPE_CAT/Diagnosi_2015-2018/Dades_XVPCA_2015-2018/"
excel.metadata<-"ESTACIONS CON CODIGO EUROPEO_revBSC_revDTES.xlsx"
site<-read_excel(paste0(path.in,excel.metadata))
site<-as.data.frame(subset(site, select = c(COD_EUROPEU, NOM_ESTACIO, LONGITUT,LATITUT,ID_TIPUS_ESTACIO,ID_TIPUS_AREA)))
site$ID_TIPUS_ESTACIO<-as.factor(site$ID_TIPUS_ESTACIO)
site$ID_TIPUS_AREA<-as.factor(site$ID_TIPUS_AREA)


for (d in 1:length(expid)){
  for (i in 1:length(pol)){
    for (j in 1:length(tb)){
      #### Reading sts files
      if (pol [i] == "O3"){varid = "sconco3"}
      if (pol [i] == "NO2"){varid = "sconcno2"}
      if (pol [i] == "PM10"){varid = "pm10"} 
      if (pol [i] == "PM2P5"){varid = "pm2p5"} 
      filein=paste0(path_in,"resultats_",varid,"_",year,"_",expid[d],".csv")
      print (filein)
      sts<-read.csv(filein,sep=";")
      #### Reading obs files
      filein=paste0(path_in,"gene_",varid,"_",year,".csv")
      obs<-read.csv(filein,sep=",")
      #### Calculate temporal coverage
      obs_cover<-sapply(obs[,-1], function(x) {(nrow(obs)-sum(is.na(x)))/nrow(obs)*100})
      obs_cover<-melt(obs_cover)
      obs_cover$codi<-rownames(obs_cover)
      #### Adding metadata info
      sts$NAME<-site$NOM_ESTACIO[match(sts$CODI,site$COD_EUROPEU)]
      sts$TIPO<-site$ID_TIPUS_ESTACIO[match(sts$CODI,site$COD_EUROPEU)]  
      sts$AREA<-site$ID_TIPUS_AREA[match(sts$CODI,site$COD_EUROPEU)]
      sts$COVER<-obs_cover$value[match(sts$CODI,obs_cover$codi)] 
      sts<-sts[order(sts$LAT),]
      sts[sapply(sts,is.numeric)]<-round(sts[sapply(sts,is.numeric)],2)
      sts<-sts[,c( colnames(sts)[which(colnames(sts) %in% c("CODI","NAME","TIPO","AREA","COVER"))],colnames(sts)[-which(colnames(sts) %in% c("CODI","NAME","TIPO","AREA","COVER"))])]
      
      col_nn<-c(paste0(pol[i],"_",tb[j],"_OBS"),paste0(pol[i],"_",tb[j],"_EXP"))
      
      sts2<-subset(sts, select=c("CODI","NAME","TIPO","AREA","COVER",col_nn))
      colnames(sts2)<-c("CODI","NAME","TIPO","AREA","COVER","OBS","MOD")
      sts2$OBS<-as.numeric(as.character(sts2$OBS))
      sts2$MOD<-as.numeric(as.character(sts2$MOD))
      sts2<-droplevels(sts2)
      sts2$NAME2<-paste0(sts2$CODI,"_",substr(sts2$NAME,1,15),"_",sts2$COVER,"%")
      sts2<-sts2[,c( colnames(sts2)[which(colnames(sts2) %in% c("NAME2"))],colnames(sts2)[-which(colnames(sts2) %in% c("NAME2"))])]
      
      if (pol[i] == "O3" && tb [j] == "8HMAX" )  { vl<-c(180)}
      if (pol[i] == "O3" && tb [j] == "PERC" )   { vl<-c(120)}
      if (pol[i] == "NO2" && tb [j] == "MEAN")   { vl<-c(40) }
      if (pol[i] == "NO2" && tb [j] == "PERC")   { vl<-c(200)}
      if (pol[i] == "PM10" && tb [j] == "MEAN")  { vl<-c(40) }
      if (pol[i] == "PM10" && tb [j] == "PERC")  { vl<-c(50) }
      if (pol[i] == "PM2P5" && tb [j] == "MEAN")  { vl<-c(20) }

      #### ploting
      file.plot<-paste0(path_out,"list_stations_",expid[d],"_ip_",pol[i],"_EIONET_XVPCA_",tb[j],"_",year,"0101_",year+1,"0101.png")
      
      plot<-ggplot() +
        geom_point(data=sts2, aes(x=factor(sts2$NAME2, level = sts2$NAME2), y=sts2$OBS,colour = "OBS"),shape =2, size=3 ) +
        geom_point(data=sts2, aes(x=factor(sts2$NAME2, level = sts2$NAME2), y=sts2$MOD,colour = "MOD"),shape=17, size=3 ) +
        geom_hline(yintercept=vl, color = "red") +
        coord_flip()+
        xlab("") +
        ylab(paste0(pol[i],"_" ,tb[j]," (ug/m3)"))+
        ggtitle (paste0(dim(sts2)," Stations - Year ", year," - ",pol[i],"_" ,tb[j]))
      ggsave(filename=file.plot, plot=plot, height = 10,dpi = 150, units = "in", device='png')
      print(file.plot) 
      #### Print csv metadata
      rownames(sts2)<-NULL
      df<-subset(sts2,select=c(CODI,NAME,TIPO,AREA,COVER))
      colnames(df)<-c("Codi","Nom de l'estació","Tipus","Àrea","Disponibilitat \n (%)")
      
      write.csv(df, 
                quote = F, row.names = T,   
                file = paste0(path_out,"tables_",pol[i],"_XVPCA_",year,".csv")                          )
      png(paste0(path_out,"tables_",pol[i],"_XVPCA_",year,".png"),
          height = 25*nrow(df), width = 200*ncol(df))
      grid.table(df)
      dev.off()
      #z = df[,5]>85
      # table(z)["TRUE"]/length(df[,5])*100  
      
    }
  }
}

