############################################################
#                                                          #
#                   Time series                            #
#                                                          #
############################################################
Sys.setlocale("LC_TIME", "en_US.UTF-8")
# We will generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))
#-------------------------------------------------------------------------------
# Plot per cada estació:
model_raw_at_AQMS <- model_raw_at_AQMS_df
hourly_pollutants <- c("no2", "o3")
daily_pollutants  <- c("pm10", "pm2p5")

if(pollutant %in% hourly_pollutants){
  plot_type <- "l"
  y_lim     <- 150
} else if(pollutant %in% daily_pollutants){
  plot_type <- "h"
  y_lim     <- 100
}

for (station_code in names_local_stations) {
  
  # station_code <- names_local_stations[[11]]
  data_single     <- data.frame(date               = df_obs_sub$date,
                                observations       = df_obs_sub[ ,station_code],
                                Raw_Model          = model_raw_at_AQMS_df[ ,station_code], 
                                UK_Model           = LOOCV_total_df[ ,station_code])
  if(execute_uk_gbm){
    data_single$GBM_Model <- LOOCV_total_gbm_df[,station_code]
    pollutant_names <- c("observations","Raw_Model","UK_Model","GBM_Model")
    name_pol        <- c("Observations", "Raw model", "UK Model", "GBM Model")
    colors          <- c("green","black","blue","red")#c("black", "black", "blue", "red", "red", "green", "green")
  } else {
    pollutant_names <- c("observations","Raw_Model","UK_Model")
    name_pol        <- c("Observations", "Raw model", "UK Model")
    colors          <- c("black", "black", "blue", "green", "green")
    
  }
  data_plot       <- data_single
  
  month_names <- c("January","February","March","April","May","June","July","August","September","October","November","December")
  if(pollutant %in% hourly_pollutants){
    for (mm in month_names) { #no2, o3 per hora o dia separar (pm treure for)
      # TIME_PLOT
      png(file = paste0(save_time_series,station_code,"_",pollutant,"_",mm, if(execute_uk_gbm) "_gbm" else "",".png"), width = 800, height = 400)
      openair::timePlot(selectByDate(data_single, year=read_year, month = mm),
                        pollutant = pollutant_names,
                        group = TRUE,                                    
                        title = station_code,
                        plot.type= plot_type,
                        auto.text= FALSE, 
                        conf.int = c(0.75, 0.99),
                        ci = FALSE,
                        ylim = c(0, y_lim),
                        ylab = paste0(pollutant, " ", "(ug/m3)"),
                        cols = c("black", "blue", "green"),
                        lty = c("dotted","solid","solid"),
                        lwd = c(3,2,2),
                        name.pol = name_pol)
      dev.off()
    }
  } else if (pollutant %in% daily_dates){
    library(ggplot2)
    
    # Configuram l'idioma d'R en anglès
    Sys.setlocale("LC_TIME", "en_US.UTF-8")
    png(file = paste0(save_time_series,station_code,"_",pollutant,"_", if(execute_uk_gbm) "_gbm" else "",".png"), width = 800, height = 400)

    ggplot(data_plot, aes(x = date)) +

      geom_point(aes(y = observations, color = "Observations"), size = 1) +
      geom_line(aes(y = observations, color = "Observations"), size = 1, linetype = "solid") +

      geom_line(aes(y = Raw_Model, color = "Raw Model"), size = 0.5, linetype = "solid") +

      geom_point(aes(y = UK_Model, color = "UK Model"), size = 0.5) +
      geom_line(aes(y = UK_Model, color = "UK Model"), size = 0.5, linetype = "solid") +
      
      # geom_point(aes(y = GBM_Model, color = "GBM Model"), size = 0.5) +
      # geom_line(aes(y = GBM_Model, color = "GBM Model"), size = 0.5, linetype = "solid") +
      
      scale_color_manual(values = colors) +
      labs(x = "", y = "Values", title = paste0("Time Series for ", pollutant, " ", read_year, " ", station_code)) +
      # ylim(0, 50) +
      scale_x_date(date_labels = "%b", date_breaks = "1 month", labels = month.name) +
      theme_minimal() +
      theme(legend.position = "bottom",
            plot.title = element_text(hjust = 0.5, size = 20, face = "bold"),
            plot.subtitle = element_text(hjust = 0.5, size = 16),
            axis.title = element_text(size = 16),
            axis.text = element_text(size = 14),
            legend.title = element_blank(),
            legend.text = element_text(size = 14),
            panel.grid.major = element_line(size = 0.2, color = "gray"),
            panel.grid.minor = element_line(size = 0.1, color = "gray"))
    
    dev.off()
    }}
  
  

#-------------------------------------------------------------------------------
# Plot per la mitjana horària de totes les estacions:
#First, we create a matrix to put the data of observations and model (corrected and raw)

#month_names = c("January","February","March","April","May","June","July","August",
              #  "September","October","November","December")

obs_total            <- matrix(nrow = length_loop, ncol = 0) 
raw_model_total      <- matrix(nrow = length_loop, ncol = 0)
corr_model_total     <- matrix(nrow = length_loop, ncol = 0)
GBM_model_total      <- matrix(nrow = length_loop, ncol = 0)

for (station_code in names_local_stations) {

  obs_estacio           <-  df_obs_sub[ ,station_code]
  raw_model_estacio     <-  model_raw_at_AQMS_df[ ,station_code]
  corr_model_estacio    <-  LOOCV_total_df[ ,station_code]
  
  obs_total             <- cbind(obs_total,obs_estacio)
  raw_model_total       <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total      <- cbind(corr_model_total,corr_model_estacio) 
  
   if(execute_gbm){
     GBM_model_total    <- cbind(GBM_model_total,LOOCV_total_gbm_df[,station_code])
   }
  
}

#Then, we calculate the means of each hour considering the whole stations
raw_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                           mod = rowMeans(raw_model_total,na.rm=TRUE),
                           model="raw_model")

corr_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                            mod = rowMeans(corr_model_total,na.rm=TRUE),
                            model="corr_model")

 GBM_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                            mod = rowMeans(GBM_model_total,na.rm=TRUE),
                            model="corr_model")

data_single   <- data.frame(date      = df_obs_sub$date,
                            observations       = raw_model$obs,
                            Model              = raw_model$mod,
                            Model_corrected    = corr_model$mod)

if(execute_gbm){
  data_single$GBM_corrected <- GBM_model$mod
  color_means     <- c("black", "red", "green")
  pollutant_names <- c("observations","Model_corrected", "GBM_corrected")
  lty_type        <- c("solid","solid","solid")
  lwd_type        <- c(3,2,2)
  name_plot       <- c("observations", "Corrected Model", "GBM Model")
} else{
  color_means     <- c("black", "blue", "green")
  pollutant_names <- c("observations","Model","Model_corrected")
  lty_type        <- c("solid","solid","dashed")
  lwd_type        <- c(3,2,2)
  name_plot       <- c("observations","Raw model", "Corrected Model")
  
  
  color_means     <- c("red", "blue")
  pollutant_names <- c("observations","Model")
  lty_type        <- c("solid","dashed")
  lwd_type        <- c(3,2)
  name_plot       <- c("observations","Raw model")
  
  color_means     <- c("red", "green")
  pollutant_names <- c("observations","Model_corrected")
  lty_type        <- c("solid","dashed")
  lwd_type        <- c(3,2)
  name_plot       <- c("observations", "Corrected Model")
}

if(pollutant %in% hourly_pollutants){
  
  png(file = paste0(save_general_statistics,"time_series_",time_resolution,"_mean_",pollutant, if(execute_uk_gbm) "_gbm" else "",".png"), width = 800, height = 400)
  openair::timePlot(data_single,#, openair::selectByDate(data_single$date, year = read_year),
                    pollutant = pollutant_names,
                    plot.type = plot_type, #for pm i li treiem els messos (for), if we want no2 or o3 plot.type="l
                    group = TRUE,
                    #y.relation = "free",
                    ylim = c(0, 150),
                    ylab= paste0(pollutant, " ", "(ug/m3)"),
                    cols = color_means,
                    lty = lty_type,
                    lwd = lwd_type,
                    name.pol = name_plot)
                    
                    
  dev.off()
} else if (pollutant %in% daily_dates){
  
  # Configuram l'idioma d'R en anglès
  Sys.setlocale("LC_TIME", "en_US.UTF-8")
  png(file = paste0(save_general_statistics,"time_series_",time_resolution,"_mean_",pollutant, if(execute_uk_gbm) "_gbm" else "",".png"), width = 800, height = 400)
  
  ggplot(data_single, aes(x = date)) +
    geom_line(aes(y = observations, color = "Observations"), size = 1) +
    
    #geom_line(aes(y = Model, color = "Raw Model"), size = 0.5, linetype = "solid") +
    
    geom_line(aes(y = Model_corrected, color = "UK Model"), size = 0.5, linetype = "solid") +
    geom_line(aes(y = GBM_corrected, color = "GBM Model"), size = 0.5, linetype = "solid") +
    
    scale_color_manual(values = color_means) +
    labs(x = "", y = "Values", title = paste0("Time Series of ", pollutant, " ", read_year)) +
    ylim(0, 50) +
    scale_x_date(date_labels = "%b", date_breaks = "1 month", labels = month.name) +
    theme_minimal() +
    theme(legend.position = "bottom",
          plot.title = element_text(hjust = 0.5, size = 20, face = "bold"),
          plot.subtitle = element_text(hjust = 0.5, size = 16),
          axis.title = element_text(size = 16),
          axis.text = element_text(size = 14),
          legend.title = element_blank(),
          legend.text = element_text(size = 14),
          panel.grid.major = element_line(size = 0.2, color = "gray"),
          panel.grid.minor = element_line(size = 0.1, color = "gray"))
  
  dev.off()
}
  
color_means <- c("blue","black","red")
  
